# Priorities in descending order. 
# Reorder to change priorities to your preference.
# Sources not in the list will get the lowest priority.
# All Installed games always have a higher priority than unistalled games.
# Only the game with the highest priority stays visible.
Priorities = [
    "Steam",
    "GOG",
    "Epic",
    "Amazon",
    "Humble",
    "Twitch",
    "Xbox",
    "Uplay",
    "Origin",
    "Battle.net",
]

# Function returning the name used for equality comparision used
# to determine if two games are the same
def clean_name(name):
    clean = name.replace('&', "and").replace('+', "and")
    return ''.join(e for e in clean.lower() if e.isalnum())

###############################################################

# from difflib import SequenceMatcher
from thread import start_new_thread

# Compute integer priority based on a game's source.
def get_source_rank(game):
    if (game.Source is not None):
        if (game.Source.Name in Priorities):
            return Priorities.index(game.Source.Name)
    return len(Priorities)

# Function called by Playnite that invokes the
# hide duplicate function.
def menu_function():
    start_new_thread(hide_duplicates, (0,))

# Function called by Playnite that invokes the
# unhide duplicate function.
def reveal_all():
    start_new_thread(unhide_all_duplicates, (0,))

# Function that returns a dictionary with clean names
# as keys and a list of games with that clean name as 
# values sorted by the value a game is assigned by key.
def create_index(key):
    games = {}
    for game in PlayniteApi.Database.Games:
        cleanName = clean_name(game.Name)
        if (cleanName not in games):
            games[cleanName] = []
        insert_sorted(games[cleanName], game, key)
    return games

# Hide all duplicates. Does not explicitly reveal the copy with
# the highest priority.
def hide_duplicates(args):
    rankRange = len(Priorities)
    index = create_index(key=lambda copy: get_source_rank(
        copy) - (rankRange + 1 if copy.IsInstalled else 0))
    PlayniteApi.Database.Games.BeginBufferUpdate()
    count = 0
    for copies in index.values():
        for i in range(1, len(copies)):
            if (not copies[i].Hidden):
                count += 1
                copies[i].Hidden = True
                PlayniteApi.Database.Games.Update(copies[i])
    PlayniteApi.Database.Games.EndBufferUpdate()
    PlayniteApi.Dialogs.ShowMessage(
        "{0} copies have been hidden.".format(count))

# Unhide all copies of games that have at least one other
# game with the same clean name.
def unhide_all_duplicates(args):
    rankRange = len(Priorities)
    index = create_index(key=lambda copy: get_source_rank(
        copy) - (rankRange + 1 if copy.IsInstalled else 0))
    PlayniteApi.Database.Games.BeginBufferUpdate()
    count = 0
    for copies in index.values():
        if (len(copies) > 1):
            for i in range(0, len(copies)):
                if (copies[i].Hidden):
                    count += 1
                    copies[i].Hidden = False
                    PlayniteApi.Database.Games.Update(copies[i])
    PlayniteApi.Database.Games.EndBufferUpdate()
    PlayniteApi.Dialogs.ShowMessage(
        "{0} copies have been revealed.".format(count))

# Inserts element into l. Assuming that l was already sorted
# by their key value, it will be sorted afterwards.
def insert_sorted(l, element, key):
    val = key(element)
    i = 0
    while i < len(l) and key(l[i]) < val:
        i += 1
    l.insert(i, element)
